### Nested Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("C:/Users/yw543_000/Desktop/Data_Clean_English.csv",header=TRUE)
Data2 <- subset(Data, Data$ODDistanceM>=10000 & Data$ODDistanceM<=12000 & Data$MainModeCode!=420 & Data$MainModeCode!=410 & Data$MainModeCode!=240)
## Count the number of Data2 rows
hh <- nrow(Data2)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(4) 

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Alternative specific constants
  b1 <- x[1]
  
  ##Time Variables
  d1 <- x[2]  #modetime
  d2 <- x[3]  #totalaccess
  
  ##Fare
  f1 <- x[4]
  
  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes
  ## (train)
  ## (bus)
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
                                            #modetime               #leisure dummy     #accesstime                                          # fare               
  train  <- Data2$ModeAvailableTrain*exp(d1*Data2$TimeTrain/100 + d2*(Data2$AccessTimeTrain +Data2$EgressTimeTrain)/100 + f1*Data2$FareTrain/1000 + b1*matrix(1,nrow =hh,ncol=1))
  car    <- Data2$ModeAvailableCar  *exp(d1*Data2$TimeCar  /100 +                                                         f1*Data2$FareCar/1000)                        
  
  ### Calculate the choice probabilities
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- car + train
  
  ## Calculate indiviudal choice probabilities
  Ptrain <- Data2$ModeAvailableTrain*(train / deno)
  Pcar   <- Data2$ModeAvailableCar  *(car   / deno)
  
  ## Avoid problems stemming from choice probabilities becoming zero
  Ptrain <- (Ptrain!=0)*Ptrain + (Ptrain==0)
  Pcar   <- (Pcar  !=0)*Pcar   + (Pcar  ==0)
  
  
  
  ## Choice results
  Ctrain   <- Data2$MainModeENG =="Rail"
  Ccar     <- Data2$MainModeENG =="Car"
  
  ## Calculate the Log-likelihood function
  LL <- colSums(Ctrain*log(Ptrain) + Ccar * log(Pcar) )
  
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr,gr=NULL, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
